@@ -1,7 +1,6 @@
 library(testthat)
-library(rlibkriging)
-
-#library(testthat)
+ Sys.setenv('OMP_THREAD_LIMIT'=2)
+ library(rlibkriging)
 
 for (kernel in c("gauss","exp")) {
 # kernel = "gauss"
